///
/// Copyright (c) 2019, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

part of haystack.hgrid;

/// HCol is a column in a HGrid.
///
/// See [Project Haystack](http://project-haystack.org/doc/Grids)
class HCol {
  /// Package private constructor
  HCol(this.index, this.name, this._meta);

//////////////////////////////////////////////////////////////////////////
// Access
//////////////////////////////////////////////////////////////////////////

  /// Returns programatic name of column.
  final String name;

  /// Col's index in a grid.
  final int index;

  /// Returns display name of column which is meta.dis or name
  String get dis {
    final id = _meta.getRef("id", checked: false);
    final dis = _meta.get("dis", checked: false);

    return (dis is HStr) ? dis.val : (id?.dis ?? name);
  }

  /// Column meta-data tags.
  HDict get meta => _meta;

//////////////////////////////////////////////////////////////////////////
// Identity
//////////////////////////////////////////////////////////////////////////

  /// Hash code is based on name and meta
  @override
  int get hashCode => _hashCode ?? (hash2(name, _meta));

  /// Equality is name and meta
  @override
  bool operator ==(Object that) {
    return identical(this, that)
          || that is HCol
          && name == that.name
          && _meta == that.meta;
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  final HDict _meta;
  int? _hashCode;
}
