///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hlist;

import 'dart:collection';

import 'package:collection/collection.dart';

import 'HVal.dart';

/// HList is an immutable list of HVal items.
class HList extends HVal with ListMixin<HVal> {
//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  static final HList EMPTY = HList([]);

  /// Create a list of the given items. The items are copied
  factory HList(Iterable<HVal?> items) => HList._(List.from(items));

  HList._(this._items);

//////////////////////////////////////////////////////////////////////////
// Access
//////////////////////////////////////////////////////////////////////////

  @override
  int get length => _items.length;
  @override
  set length(int newLength) {
    throw UnsupportedError("HList.length=");
  }

  @override
  HVal operator [](int index) => _items[index];

  @override
  void operator []=(int index, HVal value) {
    throw UnsupportedError("HList.[]=");
  }

  /// Get the number of items in the list
  int get size => _items.length;

  @override
  int get hashCode => _hashCode ?? (ListEquality<HVal>().hash(_items));
  int? _hashCode;

  /// Get the HVal at the given index
  HVal get(int i) => _items[i];

//////////////////////////////////////////////////////////////////////////
// HVal
//////////////////////////////////////////////////////////////////////////

  @override
  String toZinc() {
    StringBuffer s = StringBuffer();
    s.write('[');
    s.write(_items.map((i) => i.toZinc()).join(","));
    s.write(']');
    return s.toString();
  }

  @override
  String toJson() {
    throw UnsupportedError("HList.toJson");
  }

  @override
  bool operator ==(Object other) {
    if (runtimeType != other.runtimeType) return false;

    return identical(other, this) || ListEquality<HVal>().equals(_items, (other as HList)._items);
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  final List<HVal> _items;
}
