///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.row_test;

import 'package:haystack/haystack.dart';
import 'package:test/test.dart';

void main () {
  group('Value test', () {
    test('hval', () {
      HGridBuilder aBuilder = HGridBuilder();
      aBuilder.addCol('id');
      aBuilder.addCol('dis');
      aBuilder.addCol('area');
      aBuilder.addRow([HRef('a'), HStr('Alpha'), HNum(1200)]);
      aBuilder.addRow([HRef('b'), HStr('Beta'), null ]);

      HGridBuilder bBuilder = HGridBuilder();
      bBuilder.addCol('id');
      bBuilder.addCol('dis');
      bBuilder.addCol('area');
      bBuilder.addRow([HRef('a'), HStr('Alpha'), HNum(1200)]);
      bBuilder.addRow([HRef('b'), HStr('Beta'), aBuilder.toGrid() ]);

      HGrid bGrid = bBuilder.toGrid();
      HRow r = bGrid.row(1);

      expect(r, isNotNull);
      expect(r, equals(r));
      expect(r == r, isTrue);
    });
  });

  group('Test simple', () {
    final b = HGridBuilder();
    b.addCol('id');
    b.addCol('dis');
    b.addCol('area');
    b.addRow([HRef('a'), HStr('Alpha'), HNum(1200)]);
    b.addRow([HRef('b'), HStr('Beta'), null ]);

    final g = b.toGrid();
    final r = g.row(0);

    test('row is map', () {
        expect(r is Map, equals(true));
    });

    test('row as map', () {
      expect(r.isEmpty, equals(false));
      expect(r.isNotEmpty, equals(true));
      expect(r.containsKey("id"), equals(true));
      expect(r.containsKey("dis"), equals(true));
      expect(r["dis"].toString(), equals("Alpha"));
    });

    test('row forEach', () {
      r.forEach(( String k, HVal v ) {
        expect( r[k], equals( v ));
      });
    });
  });
}
