///
/// Copyright (c) 2019, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.zinctest;

import 'package:haystack/haystack.dart';
import 'package:test/test.dart';

void main() {
  test('', () {
    verifyGrid(
        'ver:\"2.0\"\n' +
        'fooBar33\n' +
        '\n',
        null,
        [
          "fooBar33", null
        ],
        <List<HVal>>[]
    );

    verifyGrid(
        "ver:\"2.0\" tag foo:\"bar\"\n" +
        "xyz\n" +
        "\"val\"\n" +
        "\n",
        HDictBuilder().add("tag", HMarker.VAL).add("foo", HStr("bar")).toDict(),
        ["xyz", null],
        [[ HStr("val") ]]
    );

    verifyGrid(
        "ver:\"2.0\"\n" +
        "val\n" +
        "N\n" +
        "\n",
        null,
        ["val", null],
        [[null]]
    );

    verifyGrid(
        "ver:\"2.0\"\n" +
        "a,b\n" +
        "1,2\n" +
        "3,4\n" +
        "\n",
        null,
        [
          "a", null,
          "b", null
        ],
        [
          [HNum(1.0), HNum(2.0)],
          [HNum(3.0), HNum(4.0)]
        ]
    );

    // TODO:FIXIT - HBin now encodes as 3.0, so this won't quite work.
//    verifyGrid(
//        "ver:\"2.0\" bg: Bin(image/jpeg) mark\n" +
//        "file1 dis:\"F1\" icon: Bin(image/gif),file2 icon: Bin(image/jpg)\n" +
//        "Bin(text/plain),N\n" +
//        "4,Bin(image/png)\n" +
//        "Bin(text/html; a=foo; bar=\"sep\"),Bin(text/html; charset=utf8)\n",
//        HDictBuilder().add("bg", HBin("image/jpeg")).add("mark", HMarker.VAL).toDict(),
//        [
//          "file1", HDictBuilder().add("icon", HBin("image/gif")).add("dis", HStr("F1")).toDict(),
//          "file2", HDictBuilder().add("icon", HBin("image/jpg")).toDict()
//        ],
//        [
//          [ HBin("text/plain"), null ],
//          [ HNum(4.0), HBin("image/png") ],
//          [ HBin("text/html; a=foo; bar=\"sep\""), HBin("text/html; charset=utf8") ]
//        ]
//    );

    verifyGrid(
        "ver:\"2.0\"\n" +
        "a,    b,      c,      d\n" +
        "T,    F,      N,   -99\n" +
        "2.3,  -5e-10, 2.4e20, 123e-10\n" +
        "\"\",   \"a\",   \"\\\" \\\\ \\t \\n \\r\", \"\\uabcd\"\n" +
        "`path`, @12cbb082-0c02ae73, 4s, -2.5min\n" +
        "M,R,Bin(image/png),Bin(image/png)\n" +
        "2009-12-31, 23:59:01, 01:02:03.123, 2009-02-03T04:05:06Z\n" +
        "INF, -INF, \"\", NaN\n" +
        "C(12,-34),C(0.123,-0.789),C(84.5,-77.45),C(-90,180)\n" +
        "\n",
        null, [
          "a", null,
          "b", null,
          "c", null,
          "d", null
        ], [
          [ HBool.TRUE, HBool.FALSE, null, HNum(-99.0) ],
          [ HNum(2.3), HNum(-5.0E-10), HNum(2.4E20), HNum(1.23E-8) ],
          [ HStr(""), HStr("a"), HStr("\" \\ \t \n \r"), HStr("\uabcd") ],
          [ HUri("path"), HRef("12cbb082-0c02ae73", null), HNum(4.0, "s"), HNum(-2.5, "min") ],
          [ HMarker.VAL, HRemove.VAL, HBin("image/png"), HBin("image/png") ],
          [ HDate(2009, 12, 31), HTime(23, 59, 1, 0), HTime(1, 2, 3, 123), HDateTime(HDate(2009, 2, 3),HTime(4, 5, 6, 0),HTimeZone("UTC")) ],
          [ HNum.POS_INF, HNum.NEG_INF, HStr(""), HNum.NaN ],
          [ HCoord(12.0, -34.0), HCoord(0.123, -0.789), HCoord(84.5, -77.45), HCoord(-90.0, 180.0) ]
        ]
    );

    verifyGrid(
        "ver:\"2.0\"\n" +
        "foo\n" +
        "`foo\$20bar`\n" +
        "`foo\\`bar`\n" +
        "`file \\#2`\n" +
        "\"\$15\"\n",
        null,
        [
          "foo", null,
        ],
        [
          [ HUri("foo\$20bar") ],
          [ HUri("foo`bar") ],
          [ HUri("file \\#2") ],
          [ HStr("\$15") ]
        ]
    );

    verifyGrid(
        "ver:\"2.0\"\n" +
        "a, b\n" +
        "-3.1kg,4kg\n" +
        "5%,3.2%\n" +
        "5kWh/ft\u00b2,-15kWh/m\u00b2\n" +
        "123e+12kJ/kg_dry,74\u0394\u00b0F\n",
        null,
        [
          "a", null,
          "b", null
        ],
        [
          [ HNum(-3.1, "kg"), HNum(4.0, "kg") ],
          [ HNum(5.0, "%"), HNum(3.2, "%") ],
          [ HNum(5.0, "kWh/ft\u00b2"), HNum(-15.0, "kWh/m\u00b2") ],
          [ HNum(1.23E14, "kJ/kg_dry"), HNum(74.0, "\u0394\u00b0F") ],
        ]
    );

    verifyGrid(
        "ver:\"2.0\"\n" +
        "a, b, c\n" +
        ", 1, 2\n" +
        "3, , 5\n" +
        "6, 7_000,\n" +
        ",,10\n" +
        ",,\n" +
        "14,,\n" +
        "\n",
        null,
        [
          "a", null,
          "b", null,
          "c", null,
        ],
        [
          [ null, HNum(1.0), HNum(2.0) ],
          [ HNum(3.0), null, HNum(5.0) ],
          [ HNum(6.0), HNum(7000.0), null ],
          [ null, null, HNum(10.0) ],
          [ null, null, null ],
          [ HNum(14.0), null, null ],
        ]
    );

    verifyGrid(
        "ver:\"2.0\"\n" +
        "a,b\n" +
        "2010-03-01T23:55:00.013-05:00 GMT+5,2010-03-01T23:55:00.013+10:00 GMT-10\n",
        null,
        [
          "a", null,
          "b", null,
        ],
        [
          [ HDateTime(HDate(2010, 3, 1),HTime(23, 55, 0, 13),HTimeZone("GMT+5", -18000)), HDateTime(HDate(2010, 3, 1),HTime(23, 55, 0, 13),HTimeZone("GMT-10", 36000)) ]
        ]
    );

    verifyGrid(
        "ver:\"2.0\" a: 2009-02-03T04:05:06Z foo b: 2010-02-03T04:05:06Z UTC bar c: 2009-12-03T04:05:06Z London baz\n" +
        "a\n" +
        "3.814697265625E-6\n" +
        "2010-12-18T14:11:30.924Z\n" +
        "2010-12-18T14:11:30.925Z UTC\n" +
        "2010-12-18T14:11:30.925Z London\n" +
        "45\$\n" +
        "33\u00a3\n" +
        "@12cbb08e-0c02ae73\n" +
        "7.15625E-4kWh/ft\u00b2\n" +
        "R\n" +
        "NA\n",
        HDictBuilder()
            .add("b", HDateTime(HDate(2010, 2, 3), HTime(4, 5, 6, 0), HTimeZone("UTC")))
            .add("baz", HMarker.VAL)
            .add("c", HDateTime(HDate(2009, 12, 3), HTime(4, 5, 6, 0), HTimeZone("London")))
            .add("a", HDateTime(HDate(2009, 2, 3), HTime(4, 5, 6, 0), HTimeZone("UTC")))
            .add("foo", HMarker.VAL)
            .add("bar", HMarker.VAL).toDict(),
        [
          "a", null,
        ],
        [
          [ HNum(3.814697265625E-6) ],
          [ HDateTime(HDate(2010, 12, 18),HTime(14, 11, 30, 924),HTimeZone("UTC")) ],
          [ HDateTime(HDate(2010, 12, 18),HTime(14, 11, 30, 925),HTimeZone("UTC")) ],
          [ HDateTime(HDate(2010, 12, 18),HTime(14, 11, 30, 925),HTimeZone("London")) ],
          [ HNum(45.0, "\$") ],
          [ HNum(33.0, "\u00a3") ],
          [ HRef("12cbb08e-0c02ae73", null) ],
          [ HNum(7.15625E-4, "kWh/ft\u00b2") ],
          [ HRemove.VAL ],
          [ HNA.VAL ],
        ]
    );

    verifyGrid("""ver:"2.0" tag:N
a nullmetatag:N, b markermetatag
""",
        null, // meta
        [ "a", null, "b", { "markermetatag": HMarker.VAL} ],
        []
    );
  });
}

void verifyGrid(String str, HDict? meta, List<Object?> cols, List<List<HVal?>> rows) {
/*
    System.out.println();
    System.out.println("###############################################");
    System.out.println();
    System.out.println(str);
    */

// normalize nulls
  meta ??= HDict.EMPTY;
  for (int i=0; i<cols.length; ++i) {
    if (cols[i] == null) {
      cols[i] = HDict.EMPTY;
    }
  }

// read from zinc
  HGrid grid = HZincReader(str).readGrid();
  verifyGridEq(grid, meta, cols, rows);

// write grid and verify we can parse that too
  String writeStr = HZincWriter.gridToString(grid);
  HGrid writeGrid = HZincReader(writeStr).readGrid();
  verifyGridEq(writeGrid, meta, cols, rows);
}

void verifyGridEq(HGrid grid, HDict meta, List<Object?> cols, List<List<HVal?>> rows) {
// meta
  expect(grid.meta, equals(meta));

// cols
  expect(grid.numCols, equals(cols.length/2));
  for (int i=0; i<grid.numCols; ++i) {
    expect(grid.colByIndex(i).name, equals(cols[i*2+0]));
    expect(grid.colByIndex(i).meta, equals(cols[i*2+1]));
  }

// rows
  expect(grid.numRows, equals(rows.length));
  for (int ri=0; ri<rows.length; ++ri) {
    List<HVal?> expected = rows[ri];
    HRow actual = grid.row(ri);
    for (int ci=0; ci<expected.length; ++ci) {
      expect(expected[ci], equals(actual.get(grid.colByIndex(ci).name, checked:false)));
    }
  }
}